@@ -1,16 +1,16 @@
-# Copyright 1999-2009 Gentoo Foundation
+# Copyright 1999-2015 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: JobStatusDisplay.py 15246 2010-01-29 18:47:51Z zmedico $
 
-import formatter
+from __future__ import unicode_literals
+
+import io
 import sys
 import time
 
 import portage
-from portage import StringIO
+import portage.util.formatter as formatter
 from portage import os
 from portage import _encodings
-from portage import _unicode_decode
 from portage import _unicode_encode
 from portage.output import xtermTitle
 
@@ -48,11 +48,6 @@ class JobStatusDisplay(object):
 		object.__setattr__(self, "_displayed", False)
 		object.__setattr__(self, "_last_display_time", 0)
 
-		width = portage.output.get_term_size()[1]
-		if width <= 0 or width > 80:
-			width = 80
-		object.__setattr__(self, "width", width)
-		object.__setattr__(self, "_jobs_column_width", width - 32)
 		self.reset()
 
 		isatty = os.environ.get('TERM') != 'dumb' and \
@@ -69,6 +64,20 @@ class JobStatusDisplay(object):
 			if not isinstance(v, basestring):
 				self._term_codes[k] = v.decode(encoding, 'replace')
 
+		if self._isatty:
+			width = portage.output.get_term_size()[1]
+		else:
+			width = 80
+		self._set_width(width)
+
+	def _set_width(self, width):
+		if width == getattr(self, 'width', None):
+			return
+		if width <= 0 or width > 80:
+			width = 80
+		object.__setattr__(self, "width", width)
+		object.__setattr__(self, "_jobs_column_width", width - 32)
+
 	@property
 	def out(self):
 		"""Use a lazy reference to sys.stdout, in case the API consumer has
@@ -89,7 +98,7 @@ class JobStatusDisplay(object):
 		"""
 		Initialize term control codes.
 		@rtype: bool
-		@returns: True if term codes were successfully initialized,
+		@return: True if term codes were successfully initialized,
 			False otherwise.
 		"""
 
@@ -113,7 +122,8 @@ class JobStatusDisplay(object):
 
 		term_codes = {}
 		for k, capname in self._termcap_name_map.items():
-			code = tigetstr(capname)
+			# Use _native_string for PyPy compat (bug #470258).
+			code = tigetstr(portage._native_string(capname))
 			if code is None:
 				code = self._default_term_codes[capname]
 			term_codes[k] = code
@@ -201,36 +211,38 @@ class JobStatusDisplay(object):
 	def display(self):
 		"""
 		Display status on stdout, but only if something has
-		changed since the last call.
+		changed since the last call. This always returns True,
+		for continuous scheduling via timeout_add.
 		"""
 
 		if self.quiet:
-			return
+			return True
 
 		current_time = time.time()
 		time_delta = current_time - self._last_display_time
 		if self._displayed and \
 			not self._changed:
 			if not self._isatty:
-				return
+				return True
 			if time_delta < self._min_display_latency:
-				return
+				return True
 
 		self._last_display_time = current_time
 		self._changed = False
 		self._display_status()
+		return True
 
 	def _display_status(self):
 		# Don't use len(self._completed_tasks) here since that also
 		# can include uninstall tasks.
-		curval_str = str(self.curval)
-		maxval_str = str(self.maxval)
-		running_str = str(self.running)
-		failed_str = str(self.failed)
+		curval_str = "%s" % (self.curval,)
+		maxval_str = "%s" % (self.maxval,)
+		running_str = "%s" % (self.running,)
+		failed_str = "%s" % (self.failed,)
 		load_avg_str = self._load_avg_str()
 
-		color_output = StringIO()
-		plain_output = StringIO()
+		color_output = io.StringIO()
+		plain_output = io.StringIO()
 		style_file = portage.output.ConsoleStyleFile(color_output)
 		style_file.write_listener = plain_output
 		style_writer = portage.output.StyleWriter(file=style_file, maxcol=9999)
@@ -238,36 +250,36 @@ class JobStatusDisplay(object):
 		f = formatter.AbstractFormatter(style_writer)
 
 		number_style = "INFORM"
-		f.add_literal_data(_unicode_decode("Jobs: "))
+		f.add_literal_data("Jobs: ")
 		f.push_style(number_style)
-		f.add_literal_data(_unicode_decode(curval_str))
+		f.add_literal_data(curval_str)
 		f.pop_style()
-		f.add_literal_data(_unicode_decode(" of "))
+		f.add_literal_data(" of ")
 		f.push_style(number_style)
-		f.add_literal_data(_unicode_decode(maxval_str))
+		f.add_literal_data(maxval_str)
 		f.pop_style()
-		f.add_literal_data(_unicode_decode(" complete"))
+		f.add_literal_data(" complete")
 
 		if self.running:
-			f.add_literal_data(_unicode_decode(", "))
+			f.add_literal_data(", ")
 			f.push_style(number_style)
-			f.add_literal_data(_unicode_decode(running_str))
+			f.add_literal_data(running_str)
 			f.pop_style()
-			f.add_literal_data(_unicode_decode(" running"))
+			f.add_literal_data(" running")
 
 		if self.failed:
-			f.add_literal_data(_unicode_decode(", "))
+			f.add_literal_data(", ")
 			f.push_style(number_style)
-			f.add_literal_data(_unicode_decode(failed_str))
+			f.add_literal_data(failed_str)
 			f.pop_style()
-			f.add_literal_data(_unicode_decode(" failed"))
+			f.add_literal_data(" failed")
 
 		padding = self._jobs_column_width - len(plain_output.getvalue())
 		if padding > 0:
-			f.add_literal_data(padding * _unicode_decode(" "))
+			f.add_literal_data(padding * " ")
 
-		f.add_literal_data(_unicode_decode("Load avg: "))
-		f.add_literal_data(_unicode_decode(load_avg_str))
+		f.add_literal_data("Load avg: ")
+		f.add_literal_data(load_avg_str)
 
 		# Truncate to fit width, to avoid making the terminal scroll if the
 		# line overflows (happens when the load average is large).
@@ -281,4 +293,11 @@ class JobStatusDisplay(object):
 			self._update(color_output.getvalue())
 
 		if self.xterm_titles:
-			xtermTitle(" ".join(plain_output.split()))
+			# If the HOSTNAME variable is exported, include it
+			# in the xterm title, just like emergelog() does.
+			# See bug #390699.
+			title_str = " ".join(plain_output.split())
+			hostname = os.environ.get("HOSTNAME")
+			if hostname is not None:
+				title_str = "%s: %s" % (hostname, title_str)
+			xtermTitle(title_str)
