@@ -1,8 +1,15 @@
-# Copyright 1999-2009 Gentoo Foundation
+# Copyright 1999-2012 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: create_world_atom.py 15562 2010-03-02 20:11:39Z zmedico $
 
-import portage
+import sys
+
+from portage.dep import Atom, _repo_separator
+from portage.exception import InvalidData
+
+if sys.hexversion >= 0x3000000:
+	_unicode = str
+else:
+	_unicode = unicode
 
 def create_world_atom(pkg, args_set, root_config):
 	"""Create a new atom for the world file if one does not exist.  If the
@@ -17,20 +24,38 @@ def create_world_atom(pkg, args_set, root_config):
 		return None
 	cp = arg_atom.cp
 	new_world_atom = cp
+	if arg_atom.repo:
+		new_world_atom += _repo_separator + arg_atom.repo
 	sets = root_config.sets
 	portdb = root_config.trees["porttree"].dbapi
 	vardb = root_config.trees["vartree"].dbapi
-	available_slots = set(portdb.aux_get(cpv, ["SLOT"])[0] \
-		for cpv in portdb.match(cp))
+
+	if arg_atom.repo is not None:
+		repos = [arg_atom.repo]
+	else:
+		# Iterate over portdbapi.porttrees, since it's common to
+		# tweak this attribute in order to adjust match behavior.
+		repos = []
+		for tree in portdb.porttrees:
+			repos.append(portdb.repositories.get_name_for_location(tree))
+
+	available_slots = set()
+	for cpv in portdb.match(Atom(cp)):
+		for repo in repos:
+			try:
+				available_slots.add(portdb._pkg_str(_unicode(cpv), repo).slot)
+			except (KeyError, InvalidData):
+				pass
+
 	slotted = len(available_slots) > 1 or \
 		(len(available_slots) == 1 and "0" not in available_slots)
 	if not slotted:
 		# check the vdb in case this is multislot
-		available_slots = set(vardb.aux_get(cpv, ["SLOT"])[0] \
-			for cpv in vardb.match(cp))
+		available_slots = set(vardb._pkg_str(cpv, None).slot \
+			for cpv in vardb.match(Atom(cp)))
 		slotted = len(available_slots) > 1 or \
 			(len(available_slots) == 1 and "0" not in available_slots)
-	if slotted and arg_atom != cp:
+	if slotted and arg_atom.without_repo != cp:
 		# If the user gave a specific atom, store it as a
 		# slot atom in the world file.
 		slot_atom = pkg.slot_atom
@@ -63,15 +88,27 @@ def create_world_atom(pkg, args_set, root_config):
 			# enough to identify a specific slot.
 			matches = mydb.match(arg_atom)
 			matched_slots = set()
-			for cpv in matches:
-				matched_slots.add(mydb.aux_get(cpv, ["SLOT"])[0])
+			if mydb is vardb:
+				for cpv in matches:
+					matched_slots.add(mydb._pkg_str(cpv, None).slot)
+			else:
+				for cpv in matches:
+					for repo in repos:
+						try:
+							matched_slots.add(
+								portdb._pkg_str(_unicode(cpv), repo).slot)
+						except (KeyError, InvalidData):
+							pass
+
 			if len(matched_slots) == 1:
 				new_world_atom = slot_atom
+				if arg_atom.repo:
+					new_world_atom += _repo_separator + arg_atom.repo
 
 	if new_world_atom == sets["selected"].findAtomForPackage(pkg):
 		# Both atoms would be identical, so there's nothing to add.
 		return None
-	if not slotted:
+	if not slotted and not arg_atom.repo:
 		# Unlike world atoms, system atoms are not greedy for slots, so they
 		# can't be safely excluded from world if they are slotted.
 		system_atom = sets["system"].findAtomForPackage(pkg)
