@@ -20,7 +20,7 @@ from portage.util._async.PipeLogger import PipeLogger
 from portage.util._async.PopenProcess import PopenProcess
 from _emerge.CompositeTask import CompositeTask
 
-default_hash_name = portage.const.MANIFEST2_REQUIRED_HASH
+default_hash_name = portage.const.MANIFEST2_HASH_DEFAULT
 
 # Use --no-check-certificate since Manifest digests should provide
 # enough security, and certificates can be self-signed or whatnot.
@@ -436,7 +436,7 @@ class FetchTask(CompositeTask):
 		if sys.hexversion < 0x3020000 and sys.hexversion >= 0x3000000 and \
 			not os.path.isabs(args[0]):
 			# Python 3.1 _execvp throws TypeError for non-absolute executable
-			# path passed as bytes (see http://bugs.python.org/issue8513).
+			# path passed as bytes (see https://bugs.python.org/issue8513).
 			fullname = portage.process.find_binary(args[0])
 			if fullname is None:
 				raise portage.exception.CommandNotFound(args[0])
@@ -574,7 +574,7 @@ class FetchTask(CompositeTask):
 		else:
 			for hash_name in self.digests:
 				if hash_name != "size" and \
-					hash_name in portage.checksum.hashfunc_map:
+					hash_name in portage.checksum.get_valid_checksum_keys():
 					return hash_name
 
 		return None
