@@ -1,13 +1,13 @@
-# Copyright 2007 Gentoo Foundation
+# Copyright 2007-2014 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: base.py 15717 2010-03-02 21:37:27Z zmedico $
 
 import sys
-from portage.dep import Atom, best_match_to_list, match_from_list
+from portage.dep import Atom, ExtendedAtomDict, best_match_to_list, match_from_list
 from portage.exception import InvalidAtom
 from portage.versions import cpv_getkey
 
 if sys.hexversion >= 0x3000000:
+	# pylint: disable=W0622
 	basestring = str
 
 OPERATIONS = ["merge", "unmerge"]
@@ -20,14 +20,16 @@ class PackageSet(object):
 	_operations = ["merge"]
 	description = "generic package set"
 	
-	def __init__(self):
+	def __init__(self, allow_wildcard=False, allow_repo=False):
 		self._atoms = set()
-		self._atommap = {}
+		self._atommap = ExtendedAtomDict(set)
 		self._loaded = False
 		self._loading = False
 		self.errors = []
 		self._nonatoms = set()
-		self.world_candidate = True
+		self.world_candidate = False
+		self._allow_wildcard = allow_wildcard
+		self._allow_repo = allow_repo
 
 	def __contains__(self, atom):
 		self._load()
@@ -77,10 +79,14 @@ class PackageSet(object):
 				if not a:
 					continue
 				try:
-					a = Atom(a)
+					a = Atom(a, allow_wildcard=True, allow_repo=True)
 				except InvalidAtom:
 					self._nonatoms.add(a)
 					continue
+			if not self._allow_wildcard and a.extended_syntax:
+				raise InvalidAtom("extended atom syntax not allowed here")
+			if not self._allow_repo and a.repo:
+				raise InvalidAtom("repository specification not allowed here")
 			self._atoms.add(a)
 
 		self._updateAtomMap()
@@ -110,16 +116,19 @@ class PackageSet(object):
 			self._atommap.clear()
 			atoms = self._atoms
 		for a in atoms:
-			self._atommap.setdefault(a.cp, set())
-			self._atommap[a.cp].add(a)
+			self._atommap.setdefault(a.cp, set()).add(a)
 	
 	# Not sure if this one should really be in PackageSet
-	def findAtomForPackage(self, pkg):
+	def findAtomForPackage(self, pkg, modified_use=None):
 		"""Return the best match for a given package from the arguments, or
 		None if there are no matches.  This matches virtual arguments against
 		the PROVIDE metadata.  This can raise an InvalidDependString exception
 		if an error occurs while parsing PROVIDE."""
 
+		if modified_use is not None and modified_use is not pkg.use.enabled:
+			pkg = pkg.copy()
+			pkg._metadata["USE"] = " ".join(modified_use)
+
 		# Atoms matched via PROVIDE must be temporarily transformed since
 		# match_from_list() only works correctly when atom.cp == pkg.cp.
 		rev_transform = {}
@@ -127,7 +136,7 @@ class PackageSet(object):
 			if atom.cp == pkg.cp:
 				rev_transform[atom] = atom
 			else:
-				rev_transform[Atom(atom.replace(atom.cp, pkg.cp, 1))] = atom
+				rev_transform[Atom(atom.replace(atom.cp, pkg.cp, 1), allow_wildcard=True, allow_repo=True)] = atom
 		best_match = best_match_to_list(pkg, iter(rev_transform))
 		if best_match:
 			return rev_transform[best_match]
@@ -142,12 +151,13 @@ class PackageSet(object):
 		cpv_slot_list = [pkg]
 		cp = cpv_getkey(pkg.cpv)
 		self._load() # make sure the atoms are loaded
+
 		atoms = self._atommap.get(cp)
 		if atoms:
 			for atom in atoms:
 				if match_from_list(atom, cpv_slot_list):
 					yield atom
-		provides = pkg.metadata['PROVIDE']
+		provides = pkg._metadata['PROVIDE']
 		if not provides:
 			return
 		provides = provides.split()
@@ -165,6 +175,9 @@ class PackageSet(object):
 
 class EditablePackageSet(PackageSet):
 
+	def __init__(self, allow_wildcard=False, allow_repo=False):
+		super(EditablePackageSet, self).__init__(allow_wildcard=allow_wildcard, allow_repo=allow_repo)
+		
 	def update(self, atoms):
 		self._load()
 		modified = False
@@ -172,11 +185,15 @@ class EditablePackageSet(PackageSet):
 		for a in atoms:
 			if not isinstance(a, Atom):
 				try:
-					a = Atom(a)
+					a = Atom(a, allow_wildcard=True, allow_repo=True)
 				except InvalidAtom:
 					modified = True
 					self._nonatoms.add(a)
 					continue
+			if not self._allow_wildcard and a.extended_syntax:
+				raise InvalidAtom("extended atom syntax not allowed here")
+			if not self._allow_repo and a.repo:
+				raise InvalidAtom("repository specification not allowed here")
 			normal_atoms.append(a)
 
 		if normal_atoms:
@@ -212,8 +229,14 @@ class EditablePackageSet(PackageSet):
 		raise NotImplementedError()
 
 class InternalPackageSet(EditablePackageSet):
-	def __init__(self, initial_atoms=None):
-		super(InternalPackageSet, self).__init__()
+	def __init__(self, initial_atoms=None, allow_wildcard=False, allow_repo=True):
+		"""
+		Repo atoms are allowed more often than not, so it makes sense for this
+		class to allow them by default. The Atom constructor and isvalidatom()
+		functions default to allow_repo=False, which is sufficient to ensure
+		that repo atoms are prohibited when necessary.
+		"""
+		super(InternalPackageSet, self).__init__(allow_wildcard=allow_wildcard, allow_repo=allow_repo)
 		if initial_atoms != None:
 			self.update(initial_atoms)
 
