@@ -1,4 +1,4 @@
-# Copyright 2010-2015 Gentoo Foundation
+# Copyright 2010-2018 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
 
 from __future__ import unicode_literals
@@ -296,7 +296,7 @@ def dep_eval(deplist):
 
 class _dep_choice(SlotObject):
 	__slots__ = ('atoms', 'slot_map', 'cp_map', 'all_available',
-		'all_installed_slots')
+		'all_installed_slots', 'new_slot_count')
 
 def dep_zapdeps(unreduced, reduced, myroot, use_binaries=0, trees=None):
 	"""
@@ -366,10 +366,8 @@ def dep_zapdeps(unreduced, reduced, myroot, use_binaries=0, trees=None):
 	want_update_pkg = trees[myroot].get("want_update_pkg")
 	downgrade_probe = trees[myroot].get("downgrade_probe")
 	vardb = None
-	vardb_match_pkgs = None
 	if "vartree" in trees[myroot]:
 		vardb = trees[myroot]["vartree"].dbapi
-		vardb_match_pkgs = getattr(vardb, 'match_pkgs', None)
 	if use_binaries:
 		mydbapi = trees[myroot]["bintree"].dbapi
 	else:
@@ -465,10 +463,11 @@ def dep_zapdeps(unreduced, reduced, myroot, use_binaries=0, trees=None):
 						avail_pkg = avail_pkg_use
 					avail_slot = Atom("%s:%s" % (atom.cp, avail_pkg.slot))
 
-			if vardb_match_pkgs is not None and downgrade_probe is not None:
-				inst_pkg = vardb_match_pkgs(avail_slot)
-				if (inst_pkg and avail_pkg < inst_pkg[-1] and
-					not downgrade_probe(inst_pkg[-1])):
+			if downgrade_probe is not None:
+				highest_in_slot = mydbapi_match_pkgs(avail_slot)
+				if (avail_pkg and highest_in_slot and
+					avail_pkg < highest_in_slot[-1] and
+					not downgrade_probe(avail_pkg)):
 					installed_downgrade = True
 
 			slot_map[avail_slot] = avail_pkg
@@ -499,9 +498,14 @@ def dep_zapdeps(unreduced, reduced, myroot, use_binaries=0, trees=None):
 			if current_higher or (all_match_current and not all_match_previous):
 				cp_map[avail_pkg.cp] = avail_pkg
 
+		new_slot_count = (len(slot_map) if graph_db is None else
+			sum(not graph_db.match_pkgs(slot_atom) for slot_atom in slot_map
+			if not slot_atom.cp.startswith("virtual/")))
+
 		this_choice = _dep_choice(atoms=atoms, slot_map=slot_map,
 			cp_map=cp_map, all_available=all_available,
-			all_installed_slots=False)
+			all_installed_slots=False,
+			new_slot_count=new_slot_count)
 		if all_available:
 			# The "all installed" criterion is not version or slot specific.
 			# If any version of a package is already in the graph then we
@@ -656,8 +660,8 @@ def dep_zapdeps(unreduced, reduced, myroot, use_binaries=0, trees=None):
 		if len(choices) < 2:
 			continue
 		# Prefer choices with all_installed_slots for bug #480736, and
-		# choices with a smaller number of packages for bug #632026.
-		choices.sort(key=lambda x: (not x.all_installed_slots, len(x.slot_map)))
+		# choices with a smaller number of new slots for bug #632026.
+		choices.sort(key=lambda x: (not x.all_installed_slots, x.new_slot_count))
 		for choice_1 in choices[1:]:
 			cps = set(choice_1.cp_map)
 			for choice_2 in choices:
