@@ -40,7 +40,9 @@ class MaskManager(object):
 				pmask_cache[loc] = grabfile_package(path,
 						recursive=repo_config.portage1_profiles,
 						remember_source_file=True, verify_eapi=True,
-						eapi_default=repo_config.eapi)
+						eapi_default=repo_config.eapi,
+						allow_build_id=("build-id"
+						in repo_config.profile_formats))
 				if repo_config.portage1_profiles_compat and os.path.isdir(path):
 					warnings.warn(_("Repository '%(repo_name)s' is implicitly using "
 						"'portage-1' profile format in its profiles/package.mask, but "
@@ -107,7 +109,8 @@ class MaskManager(object):
 				continue
 			repo_lines = grabfile_package(os.path.join(repo.location, "profiles", "package.unmask"), \
 				recursive=1, remember_source_file=True,
-				verify_eapi=True, eapi_default=repo.eapi)
+				verify_eapi=True, eapi_default=repo.eapi,
+				allow_build_id=("build-id" in repo.profile_formats))
 			lines = stack_lists([repo_lines], incremental=1, \
 				remember_source_file=True, warn_for_unmatched_removal=True,
 				strict_warn_for_unmatched_removal=strict_umatched_removal)
@@ -122,13 +125,15 @@ class MaskManager(object):
 				os.path.join(x.location, "package.mask"),
 				recursive=x.portage1_directories,
 				remember_source_file=True, verify_eapi=True,
-				eapi=x.eapi, eapi_default=None))
+				eapi=x.eapi, eapi_default=None,
+				allow_build_id=x.allow_build_id))
 			if x.portage1_directories:
 				profile_pkgunmasklines.append(grabfile_package(
 					os.path.join(x.location, "package.unmask"),
 					recursive=x.portage1_directories,
 					remember_source_file=True, verify_eapi=True,
-					eapi=x.eapi, eapi_default=None))
+					eapi=x.eapi, eapi_default=None,
+					allow_build_id=x.allow_build_id))
 		profile_pkgmasklines = stack_lists(profile_pkgmasklines, incremental=1, \
 			remember_source_file=True, warn_for_unmatched_removal=True,
 			strict_warn_for_unmatched_removal=strict_umatched_removal)
@@ -143,10 +148,14 @@ class MaskManager(object):
 		if user_config:
 			user_pkgmasklines = grabfile_package(
 				os.path.join(abs_user_config, "package.mask"), recursive=1, \
-				allow_wildcard=True, allow_repo=True, remember_source_file=True, verify_eapi=False)
+				allow_wildcard=True, allow_repo=True,
+				remember_source_file=True, verify_eapi=False,
+				allow_build_id=True)
 			user_pkgunmasklines = grabfile_package(
 				os.path.join(abs_user_config, "package.unmask"), recursive=1, \
-				allow_wildcard=True, allow_repo=True, remember_source_file=True, verify_eapi=False)
+				allow_wildcard=True, allow_repo=True,
+				remember_source_file=True, verify_eapi=False,
+				allow_build_id=True)
 
 		#Stack everything together. At this point, only user_pkgmasklines may contain -atoms.
 		#Don't warn for unmatched -atoms here, since we don't do it for any other user config file.
