@@ -1,7 +1,6 @@
 # test_dep_getcpv.py -- Portage Unit Testing Functionality
 # Copyright 2006 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: test_dep_getcpv.py 14229 2009-09-12 15:29:09Z zmedico $
 
 from portage.tests import TestCase
 from portage.dep import dep_getcpv
@@ -11,12 +10,14 @@ class DepGetCPV(TestCase):
 	"""
 
 	def testDepGetCPV(self):
-		
-		prefix_ops = ["<", ">", "=", "~", "<=", 
-			      ">=", "!=", "!<", "!>", "!~"]
 
-		bad_prefix_ops = [ ">~", "<~", "~>", "~<" ]
-		postfix_ops = [ ("=", "*"), ]
+		prefix_ops = [
+			"<", ">", "=", "~", "<=", 
+			">=", "!=", "!<", "!>", "!~"
+		]
+
+		bad_prefix_ops = [">~", "<~", "~>", "~<"]
+		postfix_ops = [("=", "*"),]
 
 		cpvs = ["sys-apps/portage-2.1", "sys-apps/portage-2.1",
 				"sys-apps/portage-2.1"]
@@ -27,10 +28,10 @@ class DepGetCPV(TestCase):
 					mycpv = prefix + cpv
 					if slot:
 						mycpv += slot
-					self.assertEqual( dep_getcpv( mycpv ), cpv )
+					self.assertEqual(dep_getcpv(mycpv), cpv)
 
 				for prefix, postfix in postfix_ops:
 					mycpv = prefix + cpv + postfix
 					if slot:
 						mycpv += slot
-					self.assertEqual( dep_getcpv( mycpv ), cpv )
+					self.assertEqual(dep_getcpv(mycpv), cpv)
